<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers\Admin;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Validators\Admin\AdminPlayerValidator;
use Illuminate\Support\Facades\Hash;
use App\Managers\User\UserManager;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use App\Managers\Player\PlayerManager;
use App\Managers\Position\PositionManager;
use App\Managers\PlayerPositions\PlayerPositionsManager;
use App\Managers\SchedulePlayers\SchedulePlayersManager;
use App\Managers\PlayerRatings\PlayerRatingsManager;
use App\Managers\Tactics\TacticsManager;
use App\Managers\PlayerRatingSubstitute\PlayerRatingSubstituteManager;

class AdminPlayerController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->playerValidator = new AdminPlayerValidator();
        $this->playerManager = new PlayerManager();
        $this->positionManager = new PositionManager();
        $this->playerPositionManager = new PlayerPositionsManager();
        $this->schedulePlayers = new SchedulePlayersManager();
        $this->playerRatings = new PlayerRatingsManager();
        $this->tactics = new TacticsManager();
        $this->playerRatingsSubstitute = new PlayerRatingSubstituteManager();
    }
    
    public function add(Request $request)
    {
        if($request->exists('data'))
        {
            $photo = $request->file('photo');
            $data = $request->get('data');
            if(!isset($data['position']) || empty($data['position']))
            {
                return redirect()->back()->withErrors(['errors' => 'Position is required'])->withInput();
            }
            $positions = $data['position'];
            $validation = $this->playerValidator->add($data, $photo);
            if($validation)
            {
                return redirect()->back()->withErrors($validation)->withInput();
            }
            $imageName = rand(1000000,9999999) . date('dmYHis') . '.' . $request->file('photo')->getClientOriginalExtension();
            $request->file('photo')->move(base_path() . "/public/uploads/player/", $imageName);
            $data['photo'] = $imageName;
            $data['UUID'] = $this->helper->addUuid();
            $res = $this->playerManager->add($data);
            if(!$res)
            {
                return redirect()->back()->withErrors(array('message' => 'Something went wrong'))->withInput();
            }
            
            return redirect('/admin/player');
        }
        $positions = $this->positionManager->pluck('name', 'UUID');
        return view('/admin/player/add')->with('positions', $positions);
    }
    
    public function index()
    {
        $players = $this->playerManager->with('playerPosition')->get();
        return view('/admin/player/list')->with('players', $players);
    }
    
    public function edit(Request $request, $UUID)
    {
        if($request->exists('data'))
        {
            $photo = $request->file('photo');
            
            $data = $request->get('data');
            
            if(!isset($data['position']) || empty($data['position']))
            {
                return redirect()->back()->withErrors(['errors' => 'Position is required'])->withInput();
            }
            $positions = $data['position'];
            
            $validation = $this->playerValidator->update($data, $photo, $UUID);
            if($validation)
            {
                return redirect()->back()->withInput()->withErrors($validation);
            }
            if($photo != null)
            {
                $imageName = rand(1000000,9999999) . date('dmYHis') . '.' . $request->file('photo')->getClientOriginalExtension();
                $request->file('photo')->move(base_path() . "/public/uploads/player/", $imageName);
                $data['photo'] = $imageName;
            }
           
            $res = $this->playerManager->updateData($data, $UUID);
            if(!$res)
            {
                return redirect()->back()->withInput()->withErrors(array('message' => 'Something went wrong'));
            }
            
            return redirect('/admin/player');
        }
        $positions = $this->positionManager->pluck('name', 'UUID');
        return view('/admin/player/edit')->with('UUID', $UUID)->with('positions', $positions);
    }
    
    public function currentPlayer($UUID)
    {
       
        $team = $this->playerManager->with('playerPosition', 'multiple_positions')->where('UUID', $UUID)->first()->toArray();
        
        return response()->json($team,200);
    }
    
    public function update(Request $request, $UUID)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'update');
        
        $validation = $this->teamValidator->update($data, $UUID);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->teamManager->updateData($data, $UUID);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Updated Successfully'], 200);
    }
    
    public function delete($UUID)
    {
        if(!$player = $this->playerManager->where('UUID', $UUID)->first())
        {
            return response()->json(['Player Not Found'], 500);
        }
        
        $schdulePlayers = $this->schedulePlayers->with('schedule')->where('player_id', $UUID)->get();
        if ($schdulePlayers->isNotEmpty()) {
            foreach($schdulePlayers AS $scheduledPlayer)
            {
                $scheduledPlayerID = $scheduledPlayer['UUID'];
                $this->playerRatings->where('scheduled_players_id', $scheduledPlayerID)->delete();
            }
            $this->schedulePlayers->where('player_id', $UUID)->delete();
            
//            $message = '<b>The following Schedules are associated with Player ' . $player['name'] . ' . You will have to delete the reference to ' . $player['name'] . ' from these schedule (Post Formation to Schedule) before you delete this Position.</b>';
//            $message.= "<ul>";
//            foreach($schdulePlayers AS $column)
//            {
//                $message.= "<li>" . $column->schedule->name . "</li>";
//            }
//            $message.= "</ul>";
//            
//            return response()->json([$message], 400);
//            exit;
        }
        $this->tactics->where('player_id', $UUID)->delete();
        $this->playerPositionManager->where('player_id', $UUID)->delete();
        
        
        $delete = $this->playerManager->deleteData($UUID);
        if(!$delete)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Player Deleted Successfully'], 200);
    }
    
    public function getPlayerRating($playerID)
    {
        $playerRatings = $this->playerRatings->getPlayerRatinsForEachScheuleRaw($playerID);
        $scheduledPlayerRatingsSub1 = $this->playerRatingsSubstitute->getPlayerRatinsForEachScheuleRawSub1($playerID);
        $scheduledPlayerRatingsSub2 = $this->playerRatingsSubstitute->getPlayerRatinsForEachScheuleRawSub2($playerID);
        $scheduledPlayerRatingsSub3 = $this->playerRatingsSubstitute->getPlayerRatinsForEachScheuleRawSub3($playerID);
        
        
        $count = 0;
        $totalRatings = 0;
        foreach($playerRatings AS $playerRating)
        {
            $count++;
            $totalRatings+= $playerRating->rating;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub1))
        foreach($getPlayerRatinsForEachScheuleRawSub1 AS $sub1)
        {
            $count++;
            $totalRatings+= $sub1->sub_1;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub2))
        foreach($getPlayerRatinsForEachScheuleRawSub2 AS $sub2)
        {
            $count++;
            $totalRatings+= $sub2->sub_2;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub3))
        foreach($getPlayerRatinsForEachScheuleRawSub3 AS $sub3)
        {
            $count++;
            $totalRatings+= $sub3->sub_3;
        }
        
        $averageRating = round($totalRatings / $count,2);
        echo $averageRating;
    }
    
    public function getPlayerRatingOfSchedule($scheduleID, $playerID)
    {
        $playerRatings = $this->playerRatings->getPlayerRatinsForCurrentScheuleRaw($scheduleID, $playerID);
        $scheduledPlayerRatingsSub1 = $this->playerRatingsSubstitute->getPlayerRatinsForCurrentScheuleRawSub1($scheduleID, $playerID);
        $scheduledPlayerRatingsSub2 = $this->playerRatingsSubstitute->getPlayerRatinsForCurrentScheuleRawSub2($scheduleID, $playerID);
        $scheduledPlayerRatingsSub3 = $this->playerRatingsSubstitute->getPlayerRatinsForCurrentScheuleRawSub3($scheduleID, $playerID);
        
        
        $count = 0;
        $totalRatings = 0;
        foreach($playerRatings AS $playerRating)
        {
            $count++;
            $totalRatings+= $playerRating->rating;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub1))
        foreach($getPlayerRatinsForEachScheuleRawSub1 AS $sub1)
        {
            $count++;
            $totalRatings+= $sub1->sub_1;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub2))
        foreach($getPlayerRatinsForEachScheuleRawSub2 AS $sub2)
        {
            $count++;
            $totalRatings+= $sub2->sub_2;
        }
        
        if(!empty($getPlayerRatinsForEachScheuleRawSub3))
        foreach($getPlayerRatinsForEachScheuleRawSub3 AS $sub3)
        {
            $count++;
            $totalRatings+= $sub3->sub_3;
        }
        
        $averageRating = round($totalRatings / $count,2);
        echo $averageRating;
    }
}