<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers\Admin;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Validators\TacticSuggestionValidator;
use Illuminate\Support\Facades\Hash;
use App\Managers\TacticSuggestions\TacticSuggestionsManager;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use App\Managers\Tactics\TacticsManager;
use App\Managers\Player\PlayerManager;
use App\Managers\TacticContainer\TacticContainerManager;
use Illuminate\Support\Facades\DB;

class AdminTacticController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->tacticSuggestionsValidator = new TacticSuggestionValidator();
        $this->tacticSuggestionManager = new TacticSuggestionsManager();
        $this->tacticsManager = new TacticsManager();
        $this->playerManager = new PlayerManager();
        $this->tacticContainerManager =  new TacticContainerManager();
    }
    
    public function add()
    {
        return view('/admin/decision/add');
    }
    
    public function save(Request $request)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'create');
        
        $validation = $this->decisionValidator->add($data);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->tacticSuggestionManager->add($data);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Decision Saved Successfully'], 200);
    }
    
    public function index()
    {
        $decisions = $this->tacticSuggestionManager->with('user')->select(DB::raw('count(tactic_suggestion_like_count.no_of_likes) as no_of_likes, tactic_suggestions.*'))->leftJoin('tactic_suggestion_like_count', 'tactic_suggestions.UUID', '=', 'tactic_suggestion_like_count.tactic_suggestion_id')->groupBy('tactic_suggestions.UUID')->paginate(config('settings.no_of_records'));
        return view('/admin/tactic_suggestions/list')->with('decisions', $decisions);
    }
    
    public function edit($UUID)
    {
        return view('/admin/tactic_suggestions/edit')->with('UUID', $UUID);
    }
    
    public function currentDecisionProposal($UUID)
    {
        $decision = $this->tacticSuggestionManager->where('UUID', $UUID)->first()->toArray();
        return response()->json($decision,200);
    }
    
    public function update(Request $request, $UUID)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        
        $data = $this->helper->changeSerialzeArrayValues($data, 'update');
        $data['expiry_date'] = date("Y-m-d", strtotime($data['expiry_date']));
        $validation = $this->tacticSuggestionsValidator->update($data, $UUID);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->tacticSuggestionManager->updateData($data, $UUID);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Decision Updated Successfully'], 200);
    }
    
    public function confirmProposal(Request $request)
    {
        if(!$request->exists('id'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $id = $request->get('id');
        $confirm = $this->tacticSuggestionManager->confirmProposal($id);
        if(!$confirm)
        {
             return response()->json(['Something went wrong'], 500);
        }
        return response()->json(['Proposal Confrimed'], 200);
    }
    
    public function denyProposal(Request $request)
    {
        if(!$request->exists('id'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $id = $request->get('id');
        $confirm = $this->tacticSuggestionManager->denyProposal($id);
        if(!$confirm)
        {
             return response()->json(['Something went wrong'], 500);
        }
        return response()->json(['Proposal Confrimed'], 200);
    }
    
    public function formations()
    {
        $decisions = $this->tacticContainerManager->with('user')->orderBy('created_at', 'DESC')->paginate(config('settings.no_of_records'));
        return view('/admin/tactic_suggestions/proposals')->with('decisions', $decisions);
    }
    
    public function eachFormations($UUID){
        $players = $this->playerManager->with('playerPosition')->orderBy('name')->get();
        return view('/admin/tactic_suggestions/each_proposal')->with('players', $players)->with('UUID', $UUID);
    }
    
    public function getPlayersOfTacticSuggestions($tacticContainerID)
    {
        $players = $this->tacticsManager->where('tactic_container_id',$tacticContainerID)->get();
        return response()->json($players,200);
    }
    
    public function delete($UUID)
    {
        $res = $this->tacticSuggestionManager->deleteData($UUID);
        if(!$res){
            return response()->json(['Something went wrong'], 500);
            exit;
        }
        
        return response()->json(['Tactic Suggestion Deleted'], 200);
            exit;
    }
    
    public function deleteFormationSUggestion($UUID)
    {
        $res = $this->tacticContainerManager->deleteData($UUID);
        if(!$res){
            return response()->json(['Something went wrong'], 500);
            exit;
        }
        
        return response()->json(['Formation Suggestion Deleted'], 200);
            exit;
    }
}